% deriv.m
% This function specifies the axial derivatives to be solved in the main.m
% script by calculating the first and second order radial derivatives of
% the components mass fractions, temperature, superficial velocity in the
% discretication points using specified chemical and physical properties
% as well as radial boundary conditions.

function dxdz = deriv(~, x)

global GASCONST Ncomp RP RHOcat EPS Dp Tcoolant RADIUSi MMASS CP B S zeta Tubes

r = linspace(0, RADIUSi, RP)';    % Creates an evenly distributed grid
                                  % in the radial direction

comps = [1 2 4 5 6];                  % Not integrate CO2 component

% Splitting the x variable
w = [x(1:RP) ...                      % Combined mass fraction
     x(RP+1:RP*2) ...                 % variable for use in for loops
     zeros(RP,1)...                   % where component CO2 is excluded
     x(2*RP+1:RP*3) ...               % since it has the highest overall
     x(3*RP+1:RP*4) ...               % mass fraction (to minimise errors).
     x(4*RP+1:RP*5)];

w(:,3) = 1 - ( w(:,1) + w(:,2) + w(:,4)+ w(:,5) + w(:,6) ); % CO2 placed

uz  = x(5*RP+1:RP*6);
T   = x(6*RP+1:RP*7);
p   = x(7*RP+1);
Q   = x(7*RP+2);

%% Chemical- and Physical Properties

% Mole fractions (y) from mass fractions (w)
% y(i) = w(i)/Mw(i)/(sum(w(i)/Mw(i))
%--------------------------------------------------
y = zeros(RP,Ncomp); % Preallocation
for i = 1:Ncomp
    y(:,i) =  w(:,i)/MMASS(i)./( ...
              w(:,1)./MMASS(1) + ...   % w(RP,CH4)
              w(:,2)./MMASS(2) + ...   % w(RP,CO)
              w(:,3)./MMASS(3) + ...   % w(RP,CO2)
              w(:,4)./MMASS(4) + ...   % w(RP,H2)
              w(:,5)./MMASS(5) + ...   % w(RP,H2O)
              w(:,6)./MMASS(6) );      % w(RP,N2)
end %for
               
% Average Molar Mass of Gas Mixture [kg/kmole]
% MWg = sum(yi*MMASSi) or MWg = 1/sum(wi/MMASSi)
%---------------------------------------------
MWg = y*MMASS';  % or MWg = 1./(w*(1./MMASS)');

% Gas Mixture Viscosity from component viscosities [kg/m.s]
% assuming a linear dependency of mixture VISg and component VISi
%----------------------------------------------------------------
VISi = zeros(RP,Ncomp); % preallocation
VISg = zeros(RP,1); % preallocation
for i = 1:RP
    for j = 1:Ncomp
        VISi(i,j) = B(j)*T(i)^(1.5)/(T(i)+S(j)); % Component
        VISg(i) = y(i,:)*VISi(i,:)'; % Gas Mixture
    end %for
end %for

% Gas Mixture Heat Capacity from component heat capacities [J/kg.K]
% assuming a linear dependency of mixture CPg and component CPi
%------------------------------------------------------------------
CPi = zeros(RP,Ncomp); % preallocation
CPg = zeros(RP,1); % preallocation
for i = 1:RP
    for j = 1:Ncomp
        CPi(i,j) = CP(j,1) +CP(j,2)*T(i) +CP(j,3)*T(i)^2 +CP(j,4)*T(i)^3;
        CPg(i) = y(i,:)*CPi(i,:)'/MWg(i); % Gas Mixture
    end %for
end %for

% Gas mixture density and Reynolds numbers
%-----------------------------------------------------------------
rhog = p.*MWg./(GASCONST.*T);       % Gas Mixture Density [kg/m^3]
Rep  = rhog.*uz.*Dp./VISg;          % Particle Reynolds Number [-]
Re   = rhog.*uz.*2*RADIUSi./VISg;   % Fluid Reynolds number    [-]

% Radial mass dispersion coefficient
%--------------------------------------------------------------------
Pe = 1.1*8*(2-(1-Dp/RADIUSi)^2);    % Peclet number [-]
Dr = uz*RADIUSi/Pe;                 % Radial Dispersion Coef. [m^2/s]

% Heat transfer and reaction properties, calculated by functions
%----------------------------------------------------------------------
[Ur,LAMBDAer] = heatcoef(Rep,Re,T,y,VISg,CPg); % Heat Transfer Coefficient
                                       % and Effective Radial Conductivity

[Rcomp,DELTAHr] = reaction(T,y,p);  % Component reaction rates
                                    % and Heat of Reaction

eta = effectiveness(T,y,w,p,Rcomp); % Interparticle mass transport

%% Radial 1st order derivatives
% The file dss020.m uses a forward finite difference to calculate the
% derivative of the function f with respect to the independent variable r.
% The function head: dss020(r(1), r(n), n, f, 1);

% Temperature (dT/dr)
dTdr = dss020(r(1), r(RP), RP, T, 1)' ;

% Superficial velocity (duz/dr)
duzdr = dss020(r(1), r(RP), RP, uz, 1)';

% Mass fractions (dwi/dr)
dwdr = zeros(RP,Ncomp-1); % preallocation
for j = 1:Ncomp-1
    i = comps(j);
    dwdr(:,j) =  dss020(r(1), r(RP), RP, w(:,i), 1)';
end %for

%% Radial 2nd order derivatives
% The file dss042.m uses a central finite difference to calculate the
% second derivative of f with respect to the independent variable r.
% The function head: dss020(r(1), r(n), n, f, dfdr1, BC1(1), BC2(RP));
% Insert 2 for boundary condition type to specify use of Newmann boundary
% conditions or insert 1 for Dirichlet boundary conditions.

% Temperature (d2T/dr2)
d2Tdr2 = dss042(r(1), r(RP), RP, T, dTdr, 2, 2)';

% Component mass fractions (d2wi/dr2)
d2wdr2 = zeros(RP, Ncomp-1); % preallocation
for j = 1:Ncomp-1
    i = comps(j);
    d2wdr2(:,j) = dss042(r(1), r(RP), RP, w(:,i), dwdr(:,j), 2, 2)';
end %for

%% Axial derivatives

% Pressure (dP/dz) - Ergun equation
dpdz = ergun(rhog,uz,Rep,r);

% Temperature (dT/dz)
dTdz = 1./(rhog.*CPg.*uz) .* (LAMBDAer.*((1./r).*dTdr + d2Tdr2) + ...
     (-DELTAHr.*Rcomp(:,1).*zeta*RHOcat*(1-EPS).*eta));
 
 % Component mass fractions (dwi/dz)
dwdz = zeros(RP, Ncomp); % preallocation
for j = 1:Ncomp-1
    i = comps(j);
    dwdz(:,i) = Dr./uz.*( 1./r.*dwdr(:,j) + d2wdr2(:,j) ...
      -1./T.*dTdr.*dwdr(:,j)) ...
      +Rcomp(:,i)*MMASS(i)*zeta*RHOcat*(1-EPS).*eta./(rhog.*uz);
end %for
dwdz(:,3) = -(dwdz(:,1)+dwdz(:,2)+dwdz(:,4)+dwdz(:,5)+dwdz(:,6));

% Molar mass of gas mixture (dMWg/dz)
dMWgdz = -MWg.^2 .*(dwdz*(1./MMASS)');

% Superficial velocity (duz/dz)
duzdz = (uz./T).*dTdz -(uz./p).*dpdz -(uz./MWg).*dMWgdz;

% Heat flow (dQ/dz)
dQdz = Ur.*(T(RP)-Tcoolant).*2*pi*RADIUSi*Tubes;

%% Radial boundary conditions

% Temperature at centre (1) and wall (RP)
dTdz(1)  = dTdr(1); % = 0 at r = 0
dTdz(RP) = dTdr(RP) + Ur/LAMBDAer(RP)*(T(RP)-Tcoolant); % eq. at r = RP

% Superficial Velocity at centre (1) and wall (RP)
duzdz(1)  = duzdr(1); % = 0 at r = 0
duzdz(RP) = duzdr(RP); % = 0 at r = RP

% Component Mass fraction at centre (1) and wall (RP)
for j = 1:Ncomp-1
    i = comps(j);
    dwdz(1,i)  = dwdr(1,j); % = 0 at r = 0
    dwdz(RP,i) = dwdr(RP,j); % = 0 at r = RP
end %for
dwdz(1,3)  = 0; % for CO2 % = 0 at r = 0
dwdz(RP,3) = 0; % for CO2 % = 0 at r = RP

%% Returned vector of derivatives

dxdz =  [   dwdz(:,1);
            dwdz(:,2);
            dwdz(:,4);
            dwdz(:,5);
            dwdz(:,6);
            duzdz;
            dTdz;
            dpdz;
            dQdz;       ];

end %function
