% dss020.m
%...
%... SUBROUTINE DSS020 IS AN APPLICATION OF FOURTH-ORDER DIRECTIONAL
%... DIFFERENCING IN THE NUMERICAL METHOD OF LINES.
%...
%... IT IS INTENDED SPECIFICALLY FOR THE ANALYSIS OF CONVECTIVE SYSTEMS
%... MODELLED BY FIRST-ORDER HYPERBOLIC PARTIAL DIFFERENTIAL EQUATIONS,
%... BEING:
%...
%...        U + v*U  = 0
%...        t     x
%...
%... POSITIVE v FOR MEDIUM FLOWING IN DIRECTION OF HIGHER x.
%...
%... BASED ON A SUBROUTINE CALLED DSS020 IN BOOK BY W.E.SCHIESSER CALLED
%... THE NUMERICAL METHOD OF LINES AND ALSO FOUND ON INTERNET AT:
%... https://www.lehigh.edu/~wes1/apci/28apr00.pdf
%... 
%...
%... ARGUMENT LIST
%...
%... r(1)  LEFT VALUE OF THE SPATIAL INDEPENDENT VARIABLE (INPUT)
%...
%... r(n)  RIGHT VALUE OF THE SPATIAL INDEPENDENT VARIABLE (INPUT)
%...
%... n  NUMBER OF SPATIAL GRID POINTS, INCLUDING THE END POINTS (INPUT)
%...
%... f  ONE-DIMENSIONAL ARRAY OF THE DEPENDENT VARIABLE TO BE
%...     DIFFERENTIATED (INPUT)
%...
%... fx  ONE-DIMENSIONAL ARRAY OF THE FIRST DERIVATIVE OF f (OUTPUT)
%...
%... v  INTEGER INDEX FOR THE TYPE OF BOUNDARY CONDITION, WITH VALUE:
%...      1 - FOR A DIRICHLET BOUNDARY CONDITION
%...      2 - FOR A NEUMANN BOUNDARY CONDITION

function fx = dss020(r1, rn, n, f, v)

%... GRID SPACING
    dx    = (rn-r1)/(n-1);
    rdx   = 1/(12*dx);

%... (1) FINITE DIFFERENCE APPROXIMATION FOR POSITIVE V
    if (v > 0)
        fx(1) = rdx*( -25*f(1) +48*f(2) -36*f(3) +16*f(4) -3*f(5));
        fx(2) = rdx*( -3*f(1)  -10*f(2) +18*f(3) -6*f(4)  +1*f(5));
        fx(3) = rdx*(  1*f(1)  -8*f(2)  +0*f(3)  +8*f(4)  -1*f(5));
        fx(n) = rdx*( 3*f(n-4) -16*f(n-3) +36*f(n-2) -48*f(n-1) +25*f(n));
        for i=4:(n-1)
            fx(i) = rdx*( -1*f(i-3) +6*f(i-2) -18*f(i-1) +10*f(i) ...
                +3*f(i+1));
        end %for
        
%... (2) FINITE DIFFERENCE APPROXIMATION FOR NEGATIVE V    
    else %if
      fx(1) = rdx*(-25*f(1)+48*f(2)-36*f(3)+16*f(4)- 3*f(5));
      fx(n)   = rdx*( 3*f(n-4) -16*f(n-3) +36*f(n-2) -48*f(n-1) +25*f(n));
      fx(n-1) = rdx*(-1*f(n-4) + 6*f(n-3) -18*f(n-2) +10*f(n-1) + 3*f(n));
      fx(n-2) = rdx*( 1*f(n-4) - 8*f(n-3) + 0*f(n-2) + 8*f(n-1) - 1*f(n));
        for i=2:(n-3)
            fx(i) = rdx*(-1*f(i-1) +6*f(i) -18*f(i+1) +10*f(i+2) ...
                +3*f(i+3));
        end %for
    end %if

end %function
