% dss042.m
%...
%... FUNCTION DSS042 COMPUTES A SECOND-ORDER APPROXIMATION OF A
%... SECOND-ORDER DERIVATIVE, WITH OR WITHOUT THE NORMAL DERIVATIVE
%... AT THE BOUNDARY.
%...
%... BASED ON A SUBROUTINE CALLED DSS042 IN BOOK BY W.E.SCHIESSER AND CAN
%... ALSO FOUND AT https://www.lehigh.edu/~wes1/apci/28apr00.pdf
%...
%... ARGUMENT LIST
%...
%... x(1)  LEFT VALUE OF THE SPATIAL INDEPENDENT VARIABLE (INPUT)
%...
%... x(n)  RIGHT VALUE OF THE SPATIAL INDEPENDENT VARIABLE (INPUT)
%...
%... n  NUMBER OF SPATIAL GRID POINTS, INCLUDING THE END POINTS (INPUT)
%...
%... f  ONE-DIMENSIONAL ARRAY OF THE DEPENDENT VARIABLE TO BE
%...     DIFFERENTIATED (INPUT)
%...
%... fx  ONE-DIMENSIONAL ARRAY OF THE FIRST DERIVATIVE OF f.
%...      THE END VALUES OF fx, fx(1) AND fx(N), ARE USED IN
%...      NEUMANN BOUNDARY CONDITIONS AT X = f(1) AND X = f(n),
%...      DEPENDING ON THE ARGUMENTS NL AND NU
%...
%... fxx ONE-DIMENSIONAL ARRAY OF THE SECOND DERIVATIVE OF f (OUTPUT)
%...
%... NL INTEGER INDEX FOR THE TYPE OF BOUNDARY CONDITION AT
%... X = XL (INPUT). THE ALLOWABLE VALUES ARE:
%...
%... 1 - DIRICHLET BOUNDARY CONDITION AT X = f(1)
%... (fx(1) IS NOT USED)
%...
%... 2 - NEUMANN BOUNDARY CONDITION AT X = f(1)
%... (fx(1) IS USED)
%...
%... NU INTEGER INDEX FOR THE TYPE OF BOUNDARY CONDITION AT
%... X = f(n) (INPUT). THE ALLOWABLE VALUES ARE
%...
%... 1 - DIRICHLET BOUNDARY CONDITION AT X = XU
%... (UX(N) IS NOT USED)
%...
%... 2 - NEUMANN BOUNDARY CONDITION AT X = XU
%... (UX(N) IS USED)

function fxx = dss042(x1, xn, n, f, fx, v1, vn)

%... GRID SPACING
    dx  = (xn-x1)/(n-1);
    
%... CALCULATE UXX AT THE LEFT BOUNDARY, WITHOUT UX
    if v1==1
        fxx(1) = ( (2.)*f(1) +(-5.)*f(2) +(4.)*f(3) +(-1.)*f(4) ) /(dx^2);

%... CALCULATE UXX AT THE LEFT BOUNDARY, INCLUDING UX
    elseif v1==2
        fxx(1) = ( (-7.)*f(1) +(8.)*f(2) +(-1.)*f(3) )/ (2.*dx^2) ... 
            +(-6.)*fx(1) /(2.*dx);
    end %if

%... CALCULATE UXX AT THE RIGHT BOUNDARY, WITHOUT UX
    if vn==1
        fxx(n) = ( (2.)*f(n) +(-5.)*f(n-1) +(4.)*f(n-2) +(-1.)*f(n-3) ) ...
            /(dx^2);

%... CALCULATE UXX AT THE RIGHT BOUNDARY, INCLUDING UX
    elseif vn==2
        fxx(n)=( (-7.)*f(n) +(8.)*f(n-1) +(-1.)*f(n-2))/(2.*dx^2)...
            +(6.)*fx(n) /(2.*dx);
    end %if
                
%... CALCULATE UXX AT THE INTERIOR GRID POINTS
    fxx(2:n-1) = (f(3:n) -2*f(2:n-1) +f(1:n-2) ) /dx^2;

end %function
