% factor.m
% This function calculates the effectiveness factor in two dimensions.
% The effectiveness factor takes intraparticle mass transport limitations 
% into consideration to correct for the reaction rate and reaction heat.
% The expressions used are extracted from article: Bremer, Sundmacher(2019)
% "Operation range extension via hot-spot control for catalytic CO2 
% methanation reactors" and book: Kee (2003) "Chemically reacting flow".
%
% ASSUMPTIONS APPLIED:
% - CO2 methanation proceeds as a first order reaction.
% - CO2 is the limiting compound to diffuse to the reaction site and is 
% therefore the key component for determining the effectiveness factor.
% - The catalyst particles are spherical with 100% sphericity.
% - Steady state, equimolar counterdiffusion and isothermal process.
%
% INPUT:
% T         [=] K                Temperature
% y         [=] -                Mole fraction
% w         [=] -                Mass fraction
% p         [=] Pa               Total pressure
% Rcomp     [=] kmol/kg(cat).s   Reaction rate
%
% OUTPUT:
% eta       [=] none             Effectiveness factor

function eta = effectiveness(T,y,w,p,Rcomp)

global GASCONST RP Dp MMASS Dpore tau EPSp sumny Ncomp RHOcat EPS zeta

comps = [1 2 4 5 6];

% Binary diffusion coefficients [-]
% only for CO2,CH4; CO2,CO; CO2,H2; CO2,H2O; CO2,N2
Dij = zeros(RP,Ncomp-1);
Mv = zeros(RP,1);
for j = 1:Ncomp-1
    i = comps(j);
    Mv(i) = 2*(1/MMASS(i)+1/MMASS(3))^(-1);
    Dij(:,j) = 1e-4*0.00143*T.^1.75./...
        (p*1e-5*sqrt(Mv(i))*(sumny(i)^(1/3)+sumny(3)^(1/3))^2);
end %for

% Molecular diffusivity [m2/s]
% Mixture-averaged coefficient for CO2
SUMmole = 0;
SUMmass = 0;
for j = 1:Ncomp-1
    i = comps(j);
    SUMmole =  SUMmole + y(:,i)./Dij(:,j);
    SUMmass =  SUMmass + w(:,i)./Dij(:,j);
end %for
Dm = 1./( SUMmole + y(:,3)/(1-w(:,3))*SUMmass );

% Knudsen diffusivity [m2/s]
Dk = Dpore/3 *sqrt( 8*GASCONST*T/(pi*MMASS(3)) );

% Effective diffusivity [m2/s]
De = 1./( tau/EPSp* (1./Dm + 1./Dk) );

% Thiele modulus [-]
phi = Dp/2*(sqrt((abs(Rcomp(:,3))*MMASS(3)*zeta*RHOcat*(1-EPS)*GASCONST.*T) ...
      ./(De.*p.*y(:,3))));

% Effectiveness factor [-]
eta = 3./phi.*(1./tanh(phi)-1./phi);

end
