% ergun.m
% This function computes the 1D pressure gradient in the system by using
% the area averaged superficial velocity, gas density and Reynolds number
% (Radial change is averaged since the change in r is neglectable for P)
%
% INPUT:
% rhog      [=] kg/m^3      Gas density
% uz        [=] m/s         Superficial velocity
% Rep       [=] -           Particle reynolds number
% r         [=] m           Radial coordinate
%
% OUTPUT:
% dpdz      [=] Pa/m        Pressure gradient

function dpdz = ergun(rhog,uz,Rep,r)

global EPS Dp RP

% Area averaged density, velocity and Reynolds number for Ergun equation
%-----------------------------------------------------------------------
SUMrhog  = 0;
SUMuz    = 0;
SUMRep   = 0;
SUMrhog  = SUMrhog + rhog(1)*r(1)^2;
SUMuz    = SUMuz + uz(1)*r(1)^2;
SUMRep   = SUMRep + Rep(1)*r(1)^2;

for i=2:RP
    SUMrhog  = SUMrhog + rhog(i)*(r(i)^2-r(i-1)^2);
    SUMuz    = SUMuz + uz(i)*(r(i)^2-r(i-1)^2);
    SUMRep   = SUMRep + Rep(i)*(r(i)^2-r(i-1)^2);
end %for

rhog_avg  = SUMrhog/r(RP)^2;
uz_avg    = SUMuz/r(RP)^2;
Rep_avg   = SUMRep/r(RP)^2;

% Friction factor according to parameterisation by Tallmadge [-]
% b = 150 for Ergun parameterisation
%---------------------------------------------------------------
a   = 1.75;
b   = 4.2*Rep_avg.^(5/6);
f   = (1-EPS)/EPS^3*(a+b*(1-EPS)/Rep_avg);

% The Ergun equation to be solved
%--------------------------------
dpdz  = -f*rhog_avg*uz_avg^2/Dp;

end %function
