% heatcoef.m
% This function computes the heat transfer coefficient for radial transport
% of heat from the bed to the surrounding coolant as well as the effective 
% radial conductivity that is used in the temperature boundary condition.
% INPUT:
% Rep       [=] -             Particle Reynolds number
% Re        [=] -             Reynolds number
% T         [=] K             Temperature
% y         [=] -             Mole fraction
% VISgas    [=] kg/m.s        Gas viscosity
% CPgas     [=] J/kg.K        Gas heat capacity
%
% OUTPUT:
% Ur        [=] J/m^2.s.K     Heat coefficient
% LAMBDAer  [=] J/m.s.K       Effective radial conductivity

function [Ur,LAMBDAer] = heatcoef(Rep,Re,T,y,VISg,CPg)

global Ncomp RP EPS Dp kwall RADIUSi RADIUSo LAMBDA LAMBDAcat

% Calculates the gas heat conductivity [W/m.K]
% assumed linear dependency of mixture LAMBDAg and pure LAMBDAi
Tmatrix = [ones(RP,1) T T.^2 T.^3];    % Design matrix
LAMBDAi = zeros(RP,Ncomp);             % preallocation
for i=1:Ncomp
   LAMBDAi(:,i) = Tmatrix*LAMBDA(i,:)';
end %for
LAMBDAg=diag(y*LAMBDAi');

% Prandtl number [-]
Pr = VISg.*CPg./LAMBDAg;

% Nusselt number [-]
% Nu = 1.6*Rep(RP).^(0.5).*Pr(RP).^(0.3333); % From HYSYS based on Rep
Nu = 0.683*Re(RP).^(0.466).*Pr(RP).^(0.3333); % cylinder for Re = 40-4000 from Cengel p443

% Heat transfer coefficient inside tube [W/m2.K]
hgas = Nu.*LAMBDAg(RP)./Dp; % characteristic length for particle

% Heat transfer coefficient for coolant [W/m2.K]
% for Boiling Water: 3.000 - 100.000 (engineeringtoolbox)
hcoolant = 5000;

% Overall heat transfer coefficient [W/m2.K]
Ur = ( 1/hgas +RADIUSi/kwall*log(RADIUSo/RADIUSi) ...
       +RADIUSi/(hcoolant*RADIUSo) )^(-1);
   
% Prefactors
P    = 1.0;
BETA = 1.0; % between 0.9 - 1.0
PHI  = 0.3; % read from graph

% Radial effective static conduction
ALPHArv = 0.227e-3/(1+EPS/(2*(1-EPS))*(1-P)/P)*(T/100).^3;
ALPHArs = 0.227e-3*P/(2-P)*(T/100).^3;
LAMBDAer0 = LAMBDAg.*(EPS*(1 +BETA*Dp*ALPHArv./LAMBDAg) + ...
             BETA*(1-EPS)./(1./((1/PHI +ALPHArs*Dp./LAMBDAg)) + ...
             2/3*LAMBDAg/LAMBDAcat));

% Effective radial conductivity
LAMBDAer = LAMBDAer0 +0.14.*LAMBDAg.*Rep.*Pr;

end %function
