%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%                                                                        %
%      PSEUDO-HOMOGENEOUS,TWO-DIMENSIONAL MODEL OF CO2 METHANATION       %
%            REACTION IN A FIXED BED REACTOR WITH IMPLEMENTED            %
%                  KINETICS ACCORDING TO KOSCHANY2016                    %
%                                                                        %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% main.m
% This script solves the component mass fractions (2D), Pressure (1D),
% Superficial velocity (2D) and the temperature (2D) and plots the result.
% x = variables, y = mole fraction, w = mass fraction.

clc            % clear text in command window
clear          % clear variables created in Workspace
close all      % close all opened plots
warning off    % disable warnings in command window

tic            % Begin timer for simulation


%% DATA EXTRACTION
% ------------------------------------------------------------------

global LENGTH RADIUSi Tubes Rwall EPS EPSp tau Dpore Dp RHOcat zeta ...
    LAMBDAcat kwall Tcoolant RP Ncomp MMASS GASCONST RADIUSo Dit

run('constant.m')   % Run constant file


%% FEED PROPERTIES IMPORTED FROM HYSYS
% ------------------------------------------------------------------

% Tin = getFeedProp(1,'temperature'); % [K] 
% Pin = getFeedProp(1,'pressure');    % [Pa]
% Fin = getFeedProp(1,'totalFlow');   % [mol/s]
% yin = getFeedProp(1,'fraction');    % [molfrac]

Tin = 250+273.15;          % Initial temperature                        [K]
Pin = 16e5;                % Initial pressure                          [Pa]
Fin = 108.1974;            % Initial flowrate                       [mol/s]
yin = [0.0773, 0.0026, 0.7360, 0.1838, 0.0000, 0.0003, 0.0000]; % [molfrac]
       % CH4   % H2O   % H2    % CO2   % CO    % N2    % O2


%% PARAMETERS IMPORTED FROM HYSYS
% ------------------------------------------------------------------

% LENGTH    = getParameter('Length');                       % [m]
% RADIUSi   = getParameter('Inner tube diameter')/2;        % [m]
% Tubes     = getParameter('Number of tubes');              % [-]
% Rwall     = getParameter('Wall thickness');               % [m]
% Dit       = getParameter('Inner tube distance');          % [m]
% EPS       = getParameter('Void fraction');                % [-]
% EPSp      = getParameter('Catalyst porosity');            % [-] 
% tau       = getParameter('Catalyst tortuosity');          % [-]
% Dpore     = getParameter('Catalyst pore diameter');       % [m]
% Dp        = getParameter('Catalyst diameter');            % [m]
% RHOcat    = getParameter('Catalyst density');             % [kg/m3] 
% zeta      = getParameter('Catalyst dilution factor');     % [-]
% LAMBDAcat = getParameter('Catalyst conductivity');        % [W/(m.K)]
% kwall     = getParameter('Tube heat coefficient');        % [W/(m.K)]
% Tcoolant  = getParameter('Coolant temperature')+273.15;   % [K]
% RP        = getParameter('Radial discr. points');         % [-]

LENGTH   = 2.0;         % Length of the tube                        [m]
Tubes    = 1000;        % Number of tubes in reactor                [-]
Tcoolant = 300+273.15;  % Temp. outside the tube                    [K]
kwall    = 54;          % Heat coef. for tube metal             [W/m.K]
RADIUSi  = 0.0127;      % Inner tube radius (1 inch)                [m]
Rwall    = 0.003;       % Wall thickness                            [m]
Dit      = 0.005;       % Inner-tube distance                       [m]
RHOcat    = 2300;       % Catalyst density                  [kgcat/m^3]
zeta      = 0.60;       % Catalyst dilution factor                  [-]
LAMBDAcat = 0.243;      % Catalyst conductivity                 [W/m.K]
EPS       = 0.45;       % Void fraction                             [-]
EPSp      = 0.6;        % Catalyst porosity                         [-]
tau       = 2;          % catalyst tortuosity                       [-]
Dpore     = 1.0e-8;     % Pore diameter                             [m]
Dp        = 0.003;      % Catalyst diameter                         [m]
RP        = 20;         % Number of radial discretization points    [-]

RADIUSo  = RADIUSi+Rwall;   % Outer tube radius [m]
rp  = Dp/2;                 % Catalyst radius [m]
av  = 3/rp.*(1-EPS);        % Catalyst specific surface area [m2/m3]


%% NUMERICAL GRID
% ------------------------------------------------------------------

zspan = [0, LENGTH];            % Axial integration span
r = linspace(0, RADIUSi, RP)';  % Radial points for plots


%% INITIAL CONDITIONS
% ------------------------------------------------------------------------
% The inlet conditions are used as an initial guess to solve the model
% Chosen to not include CO2 since it has the highest overall mass fraction

Tin = Tin*ones(RP,1);   % Inlet Temperature vector [K]

Uin = 4*Fin*GASCONST*Tin/(pi*(RADIUSi*2)^2*Tubes*Pin*1000); % Superficial
                                                            % velocity
                                                            % [m/s]
% Convert mole fractions (y) to mass fractions (w)
% expression: w(i) = y(i)*Mw(i)/(sum(y(i)*Mw(i))
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
Yin = [yin(1) yin(5) yin(4) yin(3) yin(2) yin(6)]; % new component order
comps = [1 2 4 5 6];  % excluding CO2, only 5 components are determined
Win = zeros(1,RP*(Ncomp-1)); % Preallocation
for j = 1:Ncomp-1
    i = comps(j);
    Win((j-1)*RP+1:j*RP) =  Yin(i).*MMASS(i)./( ...
              Yin(1).*MMASS(1) + ... % CH4
              Yin(2).*MMASS(2) + ... % CO
              Yin(3).*MMASS(3) + ... % CO2
              Yin(4).*MMASS(4) + ... % H2
              Yin(5).*MMASS(5) + ... % H2O
              Yin(6).*MMASS(6) );    % N2
end %for

Qin = 0;   % Inlet heat flow [W]

x0 = [Win'; Uin; Tin; Pin; Qin];   % initial guess in x0 vector


%% MASS MATRIX (IN RADIAL DIRECTION)
% ------------------------------------------------------------------------
% Makes a 72x72 matrix where each 10th and 11th point on the diagonal is
% a BC point (centre of tube, wall of tube) to solve the in algebriac form
% boundary conditions together with the radial direction ODEs.

massmatrix = eye(RP*7+2,RP*7+2);    % Ones on diagonal and zeros on rest
                                    % +2 for the 2D pressure and heat flow
for i = 1:7
    massmatrix((i-1)*RP+1,(i-1)*RP+1) = 0; % Each 11th diag. point =0 (BC)
    massmatrix(i*RP,i*RP) = 0;             % Each 10th diag. point =0 (BC)
end


%% ODE SOLVER
% ------------------------------------------------------------------------
% Solves the ordinary differential equations (pressure, temperature,
% heat flow, superficial velocity and component mass fractions) in 
% axial direction for each radial discretised point.

options = odeset('Mass', massmatrix, 'RelTol',1e-6, 'AbsTol',1e-6);
[z,x] = ode15s(@deriv, zspan, x0, options);

% x-variable placing:
w_CH4   =  x(:,(1-1)*RP+1:RP*1);    % (ZP, 0:10)
w_CO    =  x(:,(2-1)*RP+1:RP*2);    % (ZP, 11:20)
w_H2    =  x(:,(3-1)*RP+1:RP*3);    % (ZP, 21:30)
w_H2O   =  x(:,(4-1)*RP+1:RP*4);    % (ZP, 31:40)
w_N2    =  x(:,(5-1)*RP+1:RP*5);    % (ZP, 41:50)
uz      =  x(:,(6-1)*RP+1:RP*6);    % (ZP, 51:60)
T       =  x(:,(7-1)*RP+1:RP*7);    % (ZP, 61:70)
p       =  x(:,(8-1)*RP+1);         % (ZP, 71)
Q       =  x(:,(8-1)*RP+2);         % (ZP, 72)

toc


%% MODEL CALCULATIONS FOR PLOTTING
% ------------------------------------------------------------------------

w_CO2 = 1 - (w_CH4 + w_CO + w_H2 + w_H2O + w_N2); % CO2 solved separately

% Mass fraction to Mole fraction
% y(i) = w(i)/Mw(i)/(sum(w(i)/Mw(i))
w = cat(3, w_CH4, w_CO, w_CO2, w_H2, w_H2O, w_N2); % 3D matrix
y = zeros(size(w)); % Preallocation
for i = 1:Ncomp
    y(:,:,i) =  w(:,:,i)/MMASS(i)./( ...
                w(:,:,1)./MMASS(1) + ...   % w(ZP,RP,CH4)
                w(:,:,2)./MMASS(2) + ...   % w(ZP,RP,CO)
                w(:,:,3)./MMASS(3) + ...   % w(ZP,RP,CO2)
                w(:,:,4)./MMASS(4) + ...   % w(ZP,RP,H2)
                w(:,:,5)./MMASS(5) + ...   % w(ZP,RP,H2O)
                w(:,:,6)./MMASS(6) );      % w(ZP,RP,N2)
end %for

% Inlet and outlet specifications printing in command window
% Assuming that the radial change is not present at inlet/outlet
y_CH4   = [y(1,1,1)*100 y(end,1,1)*100] % percentage
y_CO    = [y(1,1,2)*100 y(end,1,2)*100] % percentage
y_CO2   = [y(1,1,3)*100 y(end,1,3)*100] % percentage
y_H2    = [y(1,1,4)*100 y(end,1,4)*100] % percentage
y_H2O   = [y(1,1,5)*100 y(end,1,5)*100] % percentage
y_N2    = [y(1,1,6)*100 y(end,1,6)*100] % percentage
velocity     = [uz(1,1) uz(end,1)]  % m/s
Temperature  = [T(1,1)-273.15 T(end,1)-273.15] % dC
Pressure     = [p(1)/1e5 p(end)/1e5] % bar
Heatflow     = Q(end)/1000 % kW
X_CO2        = (1 - (y(end,1,3)/y(1,1,3)))*100 % percentage
T_max        = max(max(T))-273.15 % Maximum temperature [dC]
v_min        = min(min(uz)) % Minimum velocity [m/s]
v_max        = max(max(uz)) % Maximum velocity [m/s]

% Sizing of reactor (excluding coolant space)
InnerTubeArea = pi/4*(2*RADIUSi)^2;  % Tube cross-sectional area [m2]
ReactorAcs = Tubes*(2*RADIUSo+Dit)^2*(sqrt(3)/2);   % Reactor cs area [m2]
ReactorVolume = ReactorAcs*LENGTH;                  % Reactor volume [m3]
ReactorDiameter = sqrt(4*ReactorAcs/pi);            % Reactor diameter [m]

% Flowrate over the reactor (not for each tube)
MWg(:,:) =  1./( ...  % Molar mass of gas mixture [kg/kmol]
              w(:,:,1)./MMASS(1) + ...   % w(ZP,RP,CH4)
              w(:,:,2)./MMASS(2) + ...   % w(ZP,RP,CO)
              w(:,:,3)./MMASS(3) + ...   % w(ZP,RP,CO2)
              w(:,:,4)./MMASS(4) + ...   % w(ZP,RP,H2)
              w(:,:,5)./MMASS(5) + ...   % w(ZP,RP,H2O)
              w(:,:,6)./MMASS(6) );      % w(ZP,RP,N2)
rhog = (p.*MWg)./(GASCONST.*T);                % Mixture density [kg/m3]
VolumetricFlow = uz.*3600*InnerTubeArea*Tubes; % Volumetric flowrate [m3/h]
MassFlow       = rhog.*VolumetricFlow;         % Mass flowrate [kg/h]
MolarFlow      = MassFlow./MWg;                % Molar flowrate [kmol/h]

% Rate of reaction and Reaction heat
Rcomp = zeros(length(z),RP,Ncomp);   % pre-allocation
DELTAHr = zeros(length(z),RP);       % pre-allocation
eta = zeros(length(z),RP);           % pre-allocation
ymatrix = zeros(RP,Ncomp);           % pre-allocation
wmatrix = zeros(RP,Ncomp);           % pre-allocation
for i = 1:length(z)
    for j = 1:Ncomp
        ymatrix(:,j) = y(i,:,j); % 2D matrices in z for mole fraction
        wmatrix(:,j) = w(i,:,j); % 2D matrices in z for mass fraction
    end %for
    [Rcomp(i,:,:),DELTAHr(i,:)] = reaction(T(i,:)', ymatrix, p(i));
    eta(i,:) = effectiveness(T(i,:)',ymatrix,wmatrix,p(i),Rcomp(i,:,:));
end %for
% eta
Rrxn = Rcomp(:,:,1)*zeta*RHOcat*(1-EPS).*eta; % [kmolCH4/m^3b/s]
DELTAHr = -DELTAHr(:,:)/1000.*Rcomp(:,:,1).*zeta.*RHOcat*(1-EPS).*eta; %[kW/m^3b]


%% 3D PLOTTING OF VARIABLES

az = 135;     % Azimuth angle for setting viewpoint in figures
el = 9;       % Elevation height for setting viewpoint in figures

% Choose which x-variable to plot
dataplots = 0;
% 0  = all plots below from 8
% 1  = CH4 mole fraction (y_CH4)
% 2  = CO mole fraction (y_CO)
% 3  = CO2 mole fraction (y_CO2)
% 4  = H2 mole fraction (y_H2)
% 5  = H2O mole fraction (y_H2O)
% 6  = N2 mole fraction (y_N2)
% 7  = Gas mixture molar mass (MWg)
% 8  = Component mole fractions in 2D
% 9  = Effectiveness factor (eta) in 2D
% 10 = Reaction rate (Rrxn) in 3D
% 11 = Superficial velocity (uz) in 3D
% 12 = Temperature (T) in 3D
% 13 = Pressure (P) in 2D
% 14 = Heat flowrate (Q) in 2D
% 15 = Mass/Molar/Volumetric flowrate in 2D

run('plotting.m')   % Run plotting file


%% MODEL OUTPUT
% ------------------------------------------------------------------------
% Radial change at outlet is neglectable assumed for all below
% Might want to do a area averaging???
% 
% fout = MolarFlow(end,1)*1000/3600; % [mol/s]
% yout = [y(end,1,1) y(end,1,5) y(end,1,4) y(end,1,3) abs(y(end,1,2)) y(end,1,6) yin(7)]; % [molfrac]
% Tout = T(end,1); % [K]
% pout = p(end); % [Pa]
% 
% setProduct(1,fout,yout,'temperature', Tout,'pressure',pout)
%
% setParameter('Heat flow',Heatflow)
% setParameter('CO2 conversion',X_CO2)
% setParameter('Reactor diameter',ReactorDiameter)
% setParameter('Reactor volume',ReactorVolume)
% setParameter('Maximum temperature',T_max)
% setParameter('Minimum velocity',v_min)
% setParameter('Maximum velocity',v_max)

toc % end timer for simulation
