% reaction.m
% This function calculates the reaction rates for all the components
% and the heat of the reaction in all the discretication points.
%
% INPUT:
% T         [=] K       Temperature
% y         [=] -       Mole fraction
% p         [=] Pa      Total pressure
%
% OUTPUT:
% Rcomp     [=] kmol/kgcat.s    Component reaction rate
% DELTAHr   [=] J/kmol          Total reaction heat

function [Rcomp,DELTAHr] = reaction(T,y,p)

global GASCONST Ncomp RP Aq Eq beta Tref kref EA AXref dHX Hr

% Component partial pressures [bar]
Pcomp = zeros(RP,Ncomp); % Preallocating
for i=1:Ncomp
    Pcomp(:,i) = y(:,i).*p/1e5;
end %for

Rcomp = zeros(RP,Ncomp); % Preallocating
DELTAHr = zeros(RP,1); % Preallocating
for i=1:RP
    
  % Equilibrium constant [bar^-2]
  % from HSC chemistry database
  Keq = exp(Aq)*T(i)^beta*exp(Eq/T(i));
  
  % Rate constant [kmol/bar.kgcat.s]
  % Krxn = kref.*exp(-EA./(GASCONST*T(i)));
  Krxn = kref.*exp(EA./GASCONST*(1/Tref-1/T(i)));

  % Adsorption constant [bar^-0.5]
  % Kads = AXref.*exp(-dHX./(GASCONST*T(i)));
  Kads = AXref.*exp(dHX./GASCONST*(1/Tref-1/T(i)));

  % Denominator of rate equation [-]
  DEN = 1 + Kads(1)*Pcomp(i,5)*Pcomp(i,4)^(-0.5) ...
          + Kads(2)*Pcomp(i,4)^(0.5) ...
          + Kads(3)*Pcomp(i,3)^(0.5);

  % Reaction rate equation [kmol/kgcat.s]
  Rrxn =  ( Krxn * Pcomp(i,4)^(0.5) * Pcomp(i,3)^(0.5) ...
                 * ( 1 - (Pcomp(i,1) * Pcomp(i,5)^(2)) ...
                 / ((Pcomp(i,3) * Pcomp(i,4)^(4) * Keq) ))) /(DEN)^2;

  % Production/consumption rates of the individual components 
  % [kmol/kgcat.s], which needs conversion to reactor volume basis and
  % mass basis for mass balance. Reaction: CO2 + 4 H2 = CH4 + 2 H2O
  Rcomp(i,1) = +1*Rrxn;     % CH4
  Rcomp(i,2) = 0;           % CO
  Rcomp(i,3) = -1*Rrxn;     % CO2
  Rcomp(i,4) = -4*Rrxn;     % H2
  Rcomp(i,5) = +2*Rrxn;     % H2O
  Rcomp(i,6) = 0;           % N2

  % Heat of reaction [J/kmol] from HSC chemistry database
  % needs conversion to mass basis in mass balance.
  DELTAHr(i) = ( Hr(1) +Hr(2)*T(i) +Hr(3)*T(i).^2 +Hr(4)*T(i).^3 )*1e6;

end %for

end %function
